import argparse
import torch
import os

parser = argparse.ArgumentParser()
group = parser.add_argument_group('parameters')
group.add_argument('--beta', type=float, default=1, help='beta = 1 / k_B T')
group.add_argument('--l2', type=float, default=0.0, help='weight decay')
group = parser.add_argument_group('network parameters')
group.add_argument( '--net', type=str, default='conv', choices=['conv2','conv', 'fc','mpofc','opmlp','mpomlp','mpofc3','lrfc','pepo'], help='network type')
group.add_argument('--net_depth', type=int, default=3, help='network depth')
group.add_argument('--net_width', type=int, default=16, help='network width')
group.add_argument( '--half_kernel_size', type=int, default=1, help='(kernel_size - 1) // 2') 
group.add_argument( '--dtype', type=str, default='float32', choices=['float32', 'float64'], help='dtype')
group.add_argument('--bias', action='store_true', help='use bias')
group.add_argument('--no_stdout', action='store_true', help='use bias')
group.add_argument('--res_block', action='store_true', help='use res block')
group = parser.add_argument_group('optimizer parameters')
group.add_argument( '--seed', type=int, default=0, help='random seed, 0 for randomized') 
group.add_argument( '--optimizer', type=str, default='adam', choices=['sgd', 'sgdm', 'rmsprop', 'adam', 'adam0.5'], help='optimizer')
group.add_argument( '--batch_size', type=int, default=10**2, help='number of samples')
group.add_argument( '--test_batch_size', type=int, default=10**2, help='number of samples')
group.add_argument('--lr', type=float, default=1e-4, help='learning rate')
group.add_argument('--lr_decay', type=float, default=0.5, help='rate to decay learning rate')
group.add_argument('--lr_decay_epochs', type=int, default=100, help='epochs to decay learning rate')
group = parser.add_argument_group('system parameters')
group.add_argument( '--cuda', type=int, default=-1, help='ID of GPU to use, -1 for disabled')
group.add_argument('--log_interval', type=int, default=-1, metavar='N', help='how many batches to wait before logging training status')
group.add_argument( '--chi', type=int, default=2, help='Rank in the lr or mpo representations')

group.add_argument( '--D', type=int, default=100, help='Hidden variable dimension of FC layer.')
group.add_argument( '--log', type=str, default='last_time',  help='log filename')
group.add_argument('--ingpu', action='store_false', help='store everything in GPU')
group.add_argument( '--epochs', type=int, default=1000, help='maximum number of steps')
group.add_argument( '--dims', type=str, default='4,4,4,4', help='dtype')
group.add_argument( '--channels', type=str, default='16,32', help='channels of the convolution layers')
group.add_argument( '--paddings', type=str, default='0,2', help='padding for convolution layers')
group.add_argument( '--pools', type=str, default='2,2', help='pooling sizes for convolution layers')
group.add_argument( '--size', type=int, default=28, help='size of image')
group.add_argument( '--kernel_size', type=str, default='5,5', help='convolution kernel size')
group.add_argument('--bn', action='store_true', help='use batch_norm_1d')
group.add_argument('--dropout', type=float,default =-1.0,help='dropout, negative means not active')
group.add_argument('--cpu', action='store_true', help='use batch_norm_1d')
group.add_argument('--convmpo', action='store_true', help='use mpo in conv net')
group.add_argument('--convop', action='store_true', help='use mpo in conv net')
group.add_argument('--aug', action='store_true', help='data augmentation')
group.add_argument('--aug2', action='store_true', help='data augmentation')
group.add_argument('--valid', type=int, default=10000, help='number of validations')
group.add_argument( '--save', type=str, default='', help='File name to save the best model')
group.add_argument( '--load', type=str, default='', help='File name to load the model')
group.add_argument('--cutoff', type=float,default =1.0e-15,help='svd cutoff')
group.add_argument('--cano', type=int, default=-1, help='Interval to do canonicalizaiton on MPOLinear3 class')
group.add_argument( '--dataset', type=str, default='mnist', choices=['mnist','fmnist','cifar10'],  help='Name of dataset')

args = parser.parse_args()
if args.cuda >= 0:
    os.environ['CUDA_VISIBLE_DEVICES'] = str(args.cuda)
    args.use_cuda=True
    args.device = torch.device('cuda:0')
else:
    args.use_cuda=False
    args.device = torch.device('cpu')
if args.dtype == 'float32':
    args.dtype = torch.float32
elif args.dtype == 'float64':
    args.dtype = torch.float64
else:
    raise ValueError('Unknown dtype: {}'.format(args.dtype))

if args.cpu:
    args.use_cuda=False
    args.device = torch.device('cpu')

#print("args.device=",args.device)



