import math
import torch.nn as nn
import numpy as np

import torch
from torch.nn.parameter import Parameter
from torch.nn import functional as F
from torch.nn import init
from torch.nn.modules.module import Module
from torch._jit_internal import weak_module, weak_script_method
#from TorNcon import ncon


@weak_module
class PEPO(Module):
    def __init__(self, L, d,D=2):
        """
        L*L lattice, channel d
        """
        super(PEPO, self).__init__()
        self.d = d
        self.L = L
        self.D=D
        D2=27
        print("Constructing PEPO, d=",d,"%d x %d"%(L,L))
        scale=0.1
        self.tensors = Parameter(torch.randn(L,L,d,D,D,D,D))
        self.beta=Parameter(torch.tensor([1.0]))
        self.kernel = Parameter(torch.randn(D2,D2,D2,D2,10))

        print(self)
        print("Parameters in the class")
        params = list(self.parameters())
        params = list(filter(lambda p: p.requires_grad, params))
        nparams = int(sum([np.prod(p.shape) for p in params]))
        print('Total number of trainable parameters: {}'.format(nparams))
        for param in self.parameters():
            print(type(param.data), param.size())
        print("done")
        print(self.tensors.shape)


    @weak_script_method
    def forward(self, x_input):
        tensors=torch.einsum("abcd,cdbijkl->acdijkl",x_input,self.tensors) # batch,l,l,left,down,right,up
        batch = tensors.shape[0]
        D=self.D
        L=self.L
#        factor = torch.zeros(batch
        mps_top = torch.zeros(batch,L,D,D,D,dtype=x_input.dtype,device=x_input.device) # batch,l,left,down,right
        mps_top = tensors[:,0,:,:,:,:,0]# batch,l,10,left,down,right
        mps_bottom = torch.zeros(batch,L,D,D,D,dtype=x_input.dtype,device=x_input.device) # batch,l,10,left,up,right
        mps_bottom = tensors[:,L-1,:,:,0,:,:]# batch,l,10,left,down,right
        for i in range(1,L>>1):
            mps_top_new = torch.zeros(batch,L,mps_top.shape[2]*D,D,mps_top.shape[4]*D,dtype=x_input.dtype,device=x_input.device) # batch,l,left,down,right
            mps_bottom_new = torch.zeros(batch,L,mps_bottom.shape[2]*D,D,mps_bottom.shape[4]*D,dtype=x_input.dtype,device=x_input.device) # batch,l,10,left,down,right
            for j in range(L):
                mps_top_new[:,j,:,:,:] = torch.einsum("xabc,xijkb->xaijck",mps_top[:,j,:,:,:],tensors[:,i,j,:,:,:,:]).contiguous().view(batch,mps_top.shape[2]*D,D,mps_top.shape[4]*D) # x:batch
                mps_bottom_new[:,j,:,:,:] = torch.einsum("xabc,xibjk->xiakjc",mps_bottom[:,j,:,:,:],tensors[:,L-1-i,j,:,:,:,:]).contiguous().view(batch,mps_bottom.shape[2]*D,D,mps_bottom.shape[4]*D) # x:batch
            oldshape=mps_top_new.shape
#            print("mps_top_new.norm=",mps_top_new.norm())
            mps_top_new = mps_top_new.view(mps_top_new.shape[0],-1)
            mymax = torch.max(torch.abs(mps_top_new),1,True).values
#            print("mymax=",mymax)
#            print(mps_top_new.shape,mymax.shape)
            mps_top_new = (mps_top_new/mymax).view(oldshape)
#            print("mps_top_new.norm=",mps_top_new.norm())
#            input("LLL")
            mps_top = mps_top_new


            oldshape=mps_bottom_new.shape
            mps_bottom_new = mps_bottom_new.view(mps_bottom_new.shape[0],-1)
            mymax = torch.max(torch.abs(mps_bottom_new),1,True).values
            mps_bottom_new = (mps_bottom_new/mymax).view(oldshape)
            mps_bottom = mps_bottom_new
        lmat = torch.einsum("xab,xac->xbc",mps_top[:,0,0,:,:],mps_bottom[:,0,0,:,:])
        rmat = torch.einsum("xab,xcb->xac",mps_top[:,0,:,:,0],mps_bottom[:,0,:,:,0])
#        print(lmat)
#        input("lmat")
        mymax = torch.max(torch.abs(lmat),1,True).values
        lmat = lmat/mymax
        mymax = torch.max(torch.abs(rmat),1,True).values
        rmat = rmat/mymax
        
        for j in range(1,L>>1):
            lmat = torch.einsum("xab,xacd,xbcf->xdf",lmat,mps_top[:,j,:,:,:],mps_bottom[:,j,:,:,:]).contiguous().view(batch,mps_top.shape[4],mps_bottom.shape[4])
            rmat = torch.einsum("xab,xcda,xedb->xce",lmat,mps_top[:,j,:,:,:],mps_bottom[:,j,:,:,:]).contiguous().view(batch,mps_top.shape[2],mps_bottom.shape[2])
            mymax = torch.max(torch.abs(lmat),1,True).values
            lmat = lmat/mymax
            mymax = torch.max(torch.abs(rmat),1,True).values
            rmat = rmat/mymax
#            print(lmat)
#            input("***")
        #x = torch.einsum("xab,xab->x",lmat,rmat)
        x = torch.einsum("xab,xcd,abcde->xe",lmat,rmat,self.kernel)

######   normalize x will increase the generalization power, but decrease the expressive power
#        mymax = torch.max(torch.abs(x),1,True).values 
#        x = x/mymax

#        print(x)
#        input("x")
#        print(torch.log(torch.abs(x)))
    #    print(F.log_softmax(x*self.beta.item(), dim=1))
        return F.log_softmax(x, dim=1)
#        return torch.log(torch.abs(x))







class Net(nn.Module):
    def __init__(self,args):
        super(Net, self).__init__()
        self.n_channel_1=8
        self.n_channel_2=16
        padding_1=0
        padding_2=2
        self.fc_size=6
        self.conv1 = nn.Conv2d(1, self.n_channel_1, 5, 1,padding=padding_1)
        self.conv2 = nn.Conv2d(self.n_channel_1, self.n_channel_2, 5, 1,padding=padding_2)
        self.pepo = PEPO(self.fc_size,self.n_channel_2,3)

    def forward(self, x):
        x = F.relu(self.conv1(x))
#        print(x.shape)
        x = F.max_pool2d(x, 2, 2)
#        print(x.shape)
        x = F.relu(self.conv2(x))
#        print(x.shape)
        x = F.max_pool2d(x, 2, 2) ## after this step, the data has shape [ batch, channel, 6, 6]
#        print("input to pepo",x.shape)
        x = self.pepo(x)
#        print(x.shape)
        return F.log_softmax(x, dim=1)

